package com.azt.itower.deck;

import java.awt.Image;
import java.awt.Point;
import java.util.*;
import com.azt.itower.game.ResourceConstants;
import com.azt.itower.game.Card;
import com.azt.itower.game.GameException;
import com.azt.itower.game.CardModel;
import com.azt.itower.game.IvoryTower;
import com.azt.itower.util.*;

/**
 *
 * An AbstractDeckFactory is a Factory class for Deck Factories that create card objects
 * I'ts designed having the abstract factory pattern in mind.
 * This pattern was created well after the game's initial version as a small
 * refactoring exercise by cleve gibbon. Thanks cleve!
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public abstract class AbstractDeckFactory implements DeckFactory, ResourceConstants {
    private static final int kMAX_ROWS = 4;
    private static final int kMAX_COLUMNS = 10;
    private static Hashtable fFactories;
    private Image fDeckImage;
    private Card[] fCards;
    private int fCardModelType;

    /**
     * The constructor gets called by the constructors of its concrete subclasses
     */
    protected AbstractDeckFactory() {
        fCards = new Card[kMAX_ROWS * kMAX_COLUMNS];
    }

    /**
     * Gets a factory by name. The name is the classname of the subclasses
     * @param String The ClassName of the DeckFactory that needs instantiation
     * @return DeckFactory
     * @exception GameException
     */
    public synchronized static DeckFactory getFactory(String factoryClassName) throws GameException {
        if (factoryClassName == null)
            throw new GameException("supplied factory DeckFactory name was null");
        if (fFactories == null)
            fFactories = new Hashtable();
        DeckFactory factory = (DeckFactory)fFactories.get(factoryClassName);
        if (factory == null) {
            try {
                Class factoryClass = Class.forName(factoryClassName);
                factory = (DeckFactory)factoryClass.newInstance();
                factory.init();
                fFactories.put(factoryClassName, factory);
                //GameLogger.log("created DeckFactory -> " + factoryClassName);
            }
            catch (Exception ex) {
                GameLogger.log("Problem trying create/initialise DeckFactory -> " + factoryClassName);
                throw new GameException("Problem trying create/initialise DeckFactory");
            }
        }
        return factory;
    }

    /**
     * initialises factory
     * @param String FileName of the image containing the bitmap of the card
     * @param int The Type of card model
     */
    public void init(String resourceFile, int cardModelType) throws GameException {
        if (resourceFile == null)
            throw new GameException("supplied image name was null");
        fDeckImage = IvoryTower.loadImage(resourceFile);
        fCardModelType = cardModelType;
        buildCards();
    }

    /**
     * Gets the array of cards belonging to a factory
     * @return Card[]
     */
    public Card[] getCards() {
        return fCards;
    }

    /**
     * adds Cards to a Stack
     * @param Stack The Stack will be filled with the loaded
     * Cards in order of reading
     */
    public void addCards(Stack st) {
        for (int i = 0; i < fCards.length; i++) {
            st.push(fCards[i]);
        }
    }

    /**
     * Returns the type of the cards
     */
    public int getCardModelType() {
        return fCardModelType;
    }

    /**
     * Returns the Image that contains all the cards that a deck has
     * @retun Image
     */
    protected Image getDeckImage() {

        return fDeckImage;
    }

    /**
     * Builds the cards
     *
     */
    public void buildCards() {

        int counter = 0;
        for (int y = 0; y < kMAX_ROWS; y++) {
            for (int x = 0; x < kMAX_COLUMNS; x++) {
                Point p = new Point(x, y);
                CardModel model = createCardModel(p);
                Card tCard = new Card(model, getDeckImage(), p);
                fCards[counter++] = tCard;
            }
        }

    }

    /**
     * creates the card model of a particular card
     * @param Point The coordinates where the bitmap of the card can be found
     * @return CardModel
     */
    protected abstract CardModel createCardModel(Point p);

    /**
     * testing method
     */
    public static void main(String[] args) throws Exception {
        AbstractDeckFactory.getFactory("GreenDeckFactory");
    }
}

