package com.azt.itower.game;

import java.util.*;
import com.azt.itower.util.*;

/**
 * This is an abstraction of a player. It contains the commonality between a human
 * and a computer player
 *
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public abstract class Player {
    //    protected Card fCard[];
    private int fWallSize = 100;
    private int fTowerSize = 90;
    private String fName = "Player ";
    // Current Resources
    private int fStones = 5;
    private int fMonsters = 5;
    private int fGems = 5;
    // Daily Production
    private int fMonsterProduction = 1;
    private int fStoneProduction = 1;
    private int fGemProduction = 1;
    private Vector listeners;
    private Vector fDeckListeners;
    private Hand fHand;
    protected boolean playerIsThinking;

    /**
     * Instantiates a Player with a particular name
     * @param String The name of the player
     */
    Player(String name) {
        fName = name;
        listeners = new Vector(4);
        fDeckListeners = new Vector(4);
        //       fCard = new Card[5];
        fHand = new Hand();
    }

    /**
     * Cheks the winning condition
     * @param GameCondition The Strategy object that checks the winning condition
     * @return boolean The result of the checking of the winning condition
     */
    public boolean wins(GameCondition c) {
        return (c.checkWinCondition(this));
    }

    /**
     * Cheks the losing condition
     * @param GameCondition The Strategy object that checks the loosing condition
     * @return boolean The result of the checking of the loosing condition
     */
    public boolean loses(GameCondition c) {
        return (c.checkLoseCondition(this));
    }

    /**
     * Registers game listeners to the player
     * @param GameChangeListener Registers game listeners to the player
     */
    void addGameChangeListener(GameChangeListener gcl) {
        listeners.addElement(gcl);
    }

    /**
     * Registers deck change listeners to the player
     * @param DeckChangeListener Registers deck listeners to the player
     */
    void addDeckChangeListener(DeckChangeListener gcl) {
        fDeckListeners.addElement(gcl);
    }

    /**
     * Update its views (all the game change listeners)
     */
    protected void updateViews() {
        for (Enumeration en = listeners.elements(); en.hasMoreElements(); ) {
            GameChangeListener gcl = (GameChangeListener)en.nextElement();
            gcl.stateChanged();
        }
    }

    /**
     * Update its decks (all the deck change listeners)
     */
    protected void updateDecks() {
        for (Enumeration en = fDeckListeners.elements(); en.hasMoreElements(); ) {
            DeckChangeListener dcl = (DeckChangeListener)en.nextElement();
            dcl.deckChanged();
        }
    }

    /**
     * Returns the hand of the Player
     * @return Hand Player's current hand
     */
    public Hand getHand() {
        return fHand;
    }

    /**
     * Returns the name of the Player
     * @return String The name of the player
     */
    public String getName() {
        return fName;
    }

    /**
     * Sets the name of the Player
     * @param String The name of the Player
     */
    public void setName(String Name) {
        this.fName = Name;
    }

    /**
     * executes the playing sequence
     */
    public void play() {
        //GameLogger.log("Player : " + fName + " plays");
        playerIsThinking = true;
        DeckManager dm = DeckManager.getInstance();
        fHand.refresh();
        calculateProduction();
        updateViews();
        updateDecks();
        waitToPlay();
        playCard();
    }
    /**
     * Waits to play
     */
    public abstract void waitToPlay();

    /**
     * Plays a card
     */
    public abstract void playCard();

    /**
     * Queries to find if a player has finished playing
     */
    public void hasPlayed() {
        playerIsThinking = false;
    }

    /**
     * Pause for mil milliseconds
     * @param int The amount of milliseconds for the pause
     */
    public void pause(int mil) {
        try {
            Thread.currentThread().sleep(mil);
        } catch (Exception ex) {
        }
    }

    /**
     * Calculates the daily production of resources
     */
    protected void calculateProduction() {
        fStones = fStones + fStoneProduction;
        fMonsters = fMonsters + fMonsterProduction;
        fGems = fGems + fGemProduction;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getStones() {
        return fStones;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setStones(int Stones) {
        if (Stones < 0)
            Stones = 0;
        this.fStones = Stones;
        updateViews();
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getGems() {
        return fGems;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setGems(int Gems) {
        if (Gems < 0)
            Gems = 0;
        this.fGems = Gems;
        updateViews();
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getMonsters() {
        return fMonsters;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setMonsters(int Monsters) {
        if (Monsters < 0)
            Monsters = 0;
        this.fMonsters = Monsters;
        updateViews();
    }

    /**
     * Castle Properties
     */
    public int getWallSize() {
        return fWallSize;
    }

    /**
     * Castle Properties
     */
    public void setWallSize(int fWallSize) {
        if (fWallSize < 0) {
            fTowerSize = fTowerSize + fWallSize;
            fWallSize = 0;
        }
        this.fWallSize = fWallSize;
        updateViews();
    }

    /**
     * Castle Properties
     */
    public int getTowerSize() {
        return fTowerSize;
    }

    /**
     * Castle Properties
     */
    public void setTowerSize(int TowerSize) {
        this.fTowerSize = TowerSize;
        updateViews();
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getStoneProduction() {
        return fStoneProduction;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setStoneProduction(int StoneProduction) {
        if (StoneProduction < 1)
            StoneProduction = 1;
        fStoneProduction = StoneProduction;
        updateViews();
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getGemProduction() {
        return fGemProduction;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setGemProduction(int GemProduction) {
        if (GemProduction < 1)
            GemProduction = 1;
        fGemProduction = GemProduction;
        updateViews();
    }

    /**
     * Getters and Setters for Production Properties
     */
    public int getMonsterProduction() {
        return fMonsterProduction;
    }

    /**
     * Getters and Setters for Production Properties
     */
    public void setMonsterProduction(int MonsterProduction) {
        if (MonsterProduction < 1)
            MonsterProduction = 1;
        fMonsterProduction = MonsterProduction;
        updateViews();
    }
}
